
class UnavailableCommandException(Exception):
    pass


class MartyCommandException(Exception):
    pass


class MartyConfigException(Exception):
    pass


class MartyConnectException(Exception):
    pass


class ArgumentOutOfRangeException(Exception):
    pass


class UnavailableClientTypeException(Exception):
    pass

